<?php
/**
 * @version		$Id: store.php 111 2010-09-22 16:56:17Z gfisch $
 * @package		mymuse
 * @copyright	Copyright © 2010 - Arboreta Internet Services - All rights reserved.
 * @license		GNU/GPL
 * @author		Gordon Fisch
 * @author mail	info@mymuse.ca
 * @website		http://www.mymuse.ca
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

jimport( 'joomla.application.component.model' );

class MyMuseModelStore extends Jmodel
{
	function __construct()
	{
		parent::__construct();
		$array = JRequest::getVar('cid',  0, '', 'array');
		if($array[0]){
    		$this->setId((int)$array[0]);
		}
		global $mainframe, $option;
 		// Get the pagination request variables
  		$limit = $mainframe->getUserStateFromRequest( 'global.list.limit', 'limit', $mainframe->getCfg('list_limit'), 'int' );
  		$limitstart = $mainframe->getUserStateFromRequest( $option.'limitstart', 'limitstart', 0, 'int' );
 		$this->setState('limit', $limit);
  		$this->setState('limitstart', $limitstart);
	}
	
    /**
     * Store data array
     *
     * @$item->array
     */
     var $_data = null;
 
    /**
     * Store lists array
     *
     * @array
     */
     var $_lists = array();
    
 	/**
  	 * Pagination object
	 *
	 * @$item->object
	 */
     var $_pagination = null;
  
    /**
     * Returns the query
     * @return string The query to be used to retrieve the rows from the database
     */
    function _buildQueryAll()
    {
        $query = ' SELECT * '
            . ' FROM #__mymuse_store ';
        return $query;
    }
 
    /**
     * Returns the query
     * @return string The query to be used to retrieve the rows from the database
     */
    function _buildQueryOne()
    {
        $query = ' SELECT * '
            . ' FROM #__mymuse_store WHERE id ='
            .$this->_id;
        return $query;
    }
     
    /**
     * Retrieves the store data
     * @return mixed Array of objects or one object containing the data from the database
     */
    function getData()
    {
        // Lets load the data if it doesn't already exist
        if (empty( $this->_data ))
        {
        	if(isset($this->_id))
        	{
        		
        		$query = $this->_buildQueryOne();
        		$this->_db->setQuery( $query );
            	$this->_data = $this->_db->loadObject();
        	}
        	else
        	{
        		$query = $this->_buildQueryAll();
            	$this->_data = $this->_getList( $query );
        	}

        }
 
        return $this->_data;
    }
    
    /**
     * Method to set the store identifier
     *
     * @access    public
     * @param    int Store identifier
     * @return    void
     */
    function setId($id)
    {
    	// Set id and wipe data
    	$this->_id        = $id;
    	$this->_data    = null;
    }

    
    /**
     * Method to set the store lists
     *
     * @access    public
     * @return    array
     */
    function getLists()
    {
		$lists['country']   	= array();        
		$lists['state'] 		= array();
		$lists['countrystates'] = MyMuseHelper::listCountryState($lists, $this->_data->country, $this->_data->state);

		$currencies[] = JHTML::_('select.option', '0', '- '.JText::_('Select Currency').' -');
		$query = "SELECT id, currency_code as value, currency_name as text from #__mymuse_currency ORDER BY currency_code ASC";
		$this->_db->setQuery($query);
		$currencies = array_merge($currencies, $this->_db->loadObjectList());
		$lists['currency'] = JHTML::_('select.genericlist',  $currencies, 'currency', 'class="inputbox" size="1" ', 'value', 'text', $this->_data->currency);
		return $lists;
    }
    
	function &getStoreParams()
	{
		// Initialize variables
		$params	= null;
		$item	= &$this->_data;

		$params = new JParameter( $item->params );

		$path = JPATH_BASE.DS.'components'.DS.'com_mymuse'.DS.'models'.DS.'store.xml';

		if (file_exists( $path )) {
			$xml =& JFactory::getXMLParser('Simple');
			if ($xml->loadFile($path)) {
				$document =& $xml->document;
				$params->setXML($document->getElementByPath('config/params'));
			}
		}

		return $params;
	}
    
    
    
	/**
	* Tests if item is checked out
	*
	* @access public
	* @param int A user id
	* @return boolean True if checked out
	
	*/
	function isCheckedOut( $uid=0 )
	{
        if ($this->_loadData())
        {
            if ($uid) {
                return ($this->_data->checked_out && $this->_data->checked_out != $uid);
            } else {
                return $this->_data->checked_out;
            }
        }
	}
	/**
	* Method to checkin/unlock the item
	*
	* @access public
	* @return boolean True on success
	
	*/
	function checkin()
	{
	   if ($this->_id)
	   {
	       $item = & $this->getTable();
	       if(! $item->checkin($this->_id)) {
	           $this->setError($this->_db->getErrorMsg());
	           return false;
	       }
	   }
	   return false;
	}
	
	/**
	* Method to checkout/lock the item
	*
	* @access public
	* @param int $uid User ID of the user checking the article out
	* @return boolean True on success
	
	*/
	function checkout($uid = null)
	{
	   if ($this->_id)
	   {
            // Make sure we have a user id to checkout the article with
            if (is_null($uid)) {
                $user =& JFactory::getUser();
                $uid = $user->get('id');
            }
            // Lets get to it and checkout the thing...
            $item = & $this->getTable();
            if(!$item->checkout($uid, $this->_id)) {
                $this->setError($this->_db->getErrorMsg());
                return false;
            }
            return true;
	   }
	   return false;
	}
	
	/**
	* Method to store the item
	*
	* @access public
	* @return boolean True on success
	
	*/
	function store($data)
	{

        $row =& $this->getTable();
        $user=& JFactory::getUser();
        
		if (isset($data['storeparams']) && is_array($data['storeparams']))
		{
			$data['params'] = $data['storeparams'];
		}
        // Bind the form fields to the item table
        if (!$row->bind($data)) {
            $this->setError($this->_db->getErrorMsg());
            return false;
        }
        // if new item, order last in appropriate group
        if (!$row->id) {
            $row->ordering = $row->getNextOrder( 'id' );
        }
        $isNew = true;
        // Are we saving from an item edit?
        if ($row->id) {
            $isNew = false;

        }
        
		
 
            
        // Make sure the item table is valid
        if (!$row->check()) {
            $this->setError($this->_db->getErrorMsg());
            return false;
        }
        // Store the item table to the database
        if (!$row->store()) {
            $this->setError($this->_db->getErrorMsg());
            return false;
        }
        return true;
	}
	
	/**
	* Method to remove a item
	*
	* @access public
	* @return boolean True on success
	
	*/
	function delete($cid = array())
	{
        $result = false;
        if (count( $cid ))
        {
            JArrayHelper::toInteger($cid);
            $cids = implode( ',', $cid );
            $query = 'DELETE FROM #__mymuse_store'
            . ' WHERE id IN ( '.$cids.' )';
            $this->_db->setQuery( $query );
            if(!$this->_db->query()) {
                $this->setError($this->_db->getErrorMsg());
            	return false;
            }
        }
        return true;
	}
	
	/**
	* Method to (un)publish a item
	*
	* @access public
	* @return boolean True on success
	
	*/
	function publish($cid = array(), $publish = 1)
	{
        $user =& JFactory::getUser();
        if (count( $cid ))
        {
            JArrayHelper::toInteger($cid);
            $cids = implode( ',', $cid );
            $query = 'UPDATE #__mymuse_store'
            . ' SET published = '.(int) $publish
            . ' WHERE id IN ( '.$cids.' )'
            . ' AND ( checked_out = 0 OR ( checked_out = '.(int) $user->get('id').' ) )'
            ;
            $this->_db->setQuery( $query );
            if (!$this->_db->query()) {
                $this->setError($this->_db->getErrorMsg());
                return false;
            }
        }
        return true;
	}
	/**
	* Method to move a item
	*
	* @access public
	* @return boolean True on success
	
	*/
	function move($direction)
	{
        $row =& $this->getTable();
        if (!$row->load($this->_id)) {
            $this->setError($this->_db->getErrorMsg());
            return false;
        }
        if (!$row->move( $direction, ' AND published >= 0 ' )) {
            $this->setError($this->_db->getErrorMsg());
            return false;
        }
        return true;
	}
	
	/**
	* Method to move a item
	*
	* @access public
	* @return boolean True on success
	
	*/
	function saveorder($cid = array(), $order)
	{
        $row =& $this->getTable();
        $groupings = array();
        // update ordering values
        for( $i=0; $i < count($cid); $i++ )
        {
            $row->load( (int) $cid[$i] );

            if ($row->ordering != $order[$i])
            {
                $row->ordering = $order[$i];
                if (!$row->store()) {
                    $this->setError($this->_db->getErrorMsg());
                    return false;
                }
            }
        }
        return true;
	}
	/**
	* Method to load content item data
	*
	* @access private
	* @return boolean True on success
	
	*/
	function _loadData()
	{
        // Lets load the item if it doesn't already exist
        if (empty($this->_data))
        {
            $query = 'SELECT c.*'.
            ' FROM #__mumuse_store AS c' .
            ' WHERE c.id = '.(int) $this->_id;
            $this->_db->setQuery($query);
            $this->_data = $this->_db->loadObject();
            return (boolean) $this->_data;
        }
        return true;
    }
        
        
    /**
    * Method to initialise the item data
    *
    * @access private
    * @return boolean True on success
    
    */
    function _initData()
    {
        // Lets load the item if it doesn't already exist
        if (empty($this->_data))
        {
            $item = new stdClass();

            $item->id                 		= null;
            $item->title              		= null;
            $item->name               		= null;
            $item->alias              		= null;
            $item->image              		= null;
            $item->scope              		= null;
            $item->image_position     		= null;
            $item->description        		= null;
            $item->published          		= null;
            $item->checked_out        		= 0;
            $item->checked_out_time   		= 0;
            $item->ordering           		= null;
            $item->access             		= null;
            $item->params             		= null;
            $item->contact_last_name    	= null;
            $item->contact_first_name   	= null;
            $item->contact_title        	= null;
            $item->contact_email        	= null;
            $item->phone             		= null;
            $item->fax            			= null;
            $item->address_1            	= null;
            $item->address_2            	= null;
            $item->city            		= null;
            $item->state            		= null;
            $item->country            		= null;
            $item->zip             		= null;
            $item->store_thumb_image       = null;
            $item->store_full_image        = null;
            $item->currency             	= null;
            $item->player					= null;
            $item->my_reviews_catid		= null;
			$item->my_section_artists		= null;
			$item->my_section_catalog		= null;
            
            $this->_data = $item;
            return (boolean) $this->_data;
        }
        return true;
	}

}
