<?php
/**
 * @version		$Id: product.php 157 2010-10-12 18:07:17Z gfisch $
 * @package		mymuse
 * @copyright	Copyright © 2010 - Arboreta Internet Services - All rights reserved.
 * @license		GNU/GPL
 * @author		Gordon Fisch
 * @author mail	info@mymuse.ca
 * @website		http://www.mymuse.ca
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die( 'Restricted access' );

jimport('joomla.application.component.model');

/**
 * MyMuse Product Model
 *
 * @package		Joomla
 * @subpackage	MyMuse
 * @since 1.5
 */
class MyMuseModelProduct extends JModel
{

	/**
	 * name
	 *
	 * @var string
	 */
	var $name = 'product';
	
	/**
	 * Product data
	 *
	 * @var object
	 */
	var $_product = null;
	
	/**
	 * Product id
	 *
	 * @var int
	 */
	var $_id = null;

	/**
	 * Constructor
	 *
	 * @since 1.5
	 */
	function __construct()
	{
		parent::__construct();

		$id = JRequest::getVar('id', 0, '', 'int');
		$this->setId((int)$id);
	}

	/**
	 * Method to set the product id
	 *
	 * @access	public
	 * @param	int	Product ID number
	 */
	function setId($id)
	{
		// Set new product ID and wipe data
		$this->_id		= $id;
		$this->_product	= null;
	}

	/**
	 * Overridden set method to pass properties on to the product
	 *
	 * @access	public
	 * @param	string	$property	The name of the property
	 * @param	mixed	$value		The value of the property to set
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function set( $property, $value=null )
	{
		if ($this->_loadProduct()) {
			$this->_product->$property = $value;
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Overridden get method to get properties from the product
	 *
	 * @access	public
	 * @param	string	$property	The name of the property
	 * @param	mixed	$value		The value of the property to set
	 * @return 	mixed 				The value of the property
	 * @since	1.5
	 */
	function get($property, $default=null)
	{
		if ($this->_loadProduct()) {
			if(isset($this->_product->$property)) {
				return $this->_product->$property;
			}
		}
		return $default;
	}

	/**
	 * Method to get mymuse product data
	 *
	 * @since 1.5
	 */
	function &getProduct()
	{
		// Load the Product data
		if ($this->_loadProduct())
		{
			$user	= & JFactory::getUser();

			// Is the category published?
			if (!$this->_product->cat_pub && $this->_product->catid) {
				JError::raiseError( 404, JText::_("MYMUSE RESOURCE NOT FOUND") );
			}

			// Is the section published?
			if ($this->_product->sectionid)
			{
				if ($this->_product->sec_pub === null)
				{
					// probably a new item
					// check the sectionid probably passed in the request
					$db =& $this->getDBO();
					$query = 'SELECT published' .
							' FROM #__sections' .
							' WHERE id = ' . (int) $this->_product->sectionid;
					$db->setQuery( $query );
					$this->_product->sec_pub = $db->loadResult();
				}
				if (!$this->_product->sec_pub)
				{
					JError::raiseError( 404, JText::_("MYMUSE RESOURCE NOT FOUND") );
				}
			}

			// Do we have access to the category?
			if (($this->_product->cat_access > $user->get('aid', 0)) && $this->_product->catid) {
				JError::raiseError( 403, JText::_("MYMUSE ALERTNOTAUTH") );
			}

			// Do we have access to the section?
			if (($this->_product->sec_access > $user->get('aid', 0)) && $this->_product->sectionid) {
				JError::raiseError( 403, JText::_("MYMUSE ALERTNOTAUTH") );
			}

			$this->_loadProductParams();
		}
		else
		{
			$user =& JFactory::getUser();
			$product =& JTable::getInstance('content');
			$product->state			= 1;
			$product->cat_pub		= null;
			$product->sec_pub		= null;
			$product->cat_access	= null;
			$product->sec_access	= null;
			$product->author		= null;
			$product->created_by	= $user->get('id');
			$product->parameters	= new JParameter( '' );
			$product->text			= '';
			$this->_product			= $product;
		}

		return $this->_product;
	}

	/**
	 * Method to increment the hit counter for the product
	 *
	 * @access	public
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function hit()
	{
		global $mainframe;

		if ($this->_id)
		{
			require_once( MYMUSE_ADMIN_PATH.DS.'tables'.DS.'product.php' );
			$product = new TableProduct($this->_db);
			$product->load( $this->_id );

			$product->hits++;
			$product->store();
			return true;
		}
		return false;
	}



	/**
	 * Method to load content product data
	 *
	 * @access	private
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function _loadProduct()
	{
		global $mainframe,$params; 

		if($this->_id == '0')
		{
			return false;
		}

		// Load the product if it doesn't already exist
		if (empty($this->_product))
		{
			global $params;
			// Get the WHERE clause
			$where	= $this->_buildContentWhere();

			$query = 'SELECT a.*, u.name AS author, u.usertype, cc.title AS category, cc.id as catid, s.title AS section,' .
					' CASE WHEN CHAR_LENGTH(a.alias) THEN CONCAT_WS(":", a.id, a.alias) ELSE a.id END as slug,'.
					' CASE WHEN CHAR_LENGTH(cc.alias) THEN CONCAT_WS(":", cc.id, cc.alias) ELSE cc.id END as catslug,'.
					' g.name AS groups, s.published AS sec_pub, cc.published AS cat_pub, s.access AS sec_access, cc.access AS cat_access '.
					' FROM #__mymuse_product AS a' .
					' LEFT JOIN #__mymuse_product_category_xref AS cxref ON cxref.product_id = a.id' .
					' LEFT JOIN #__mymuse_categories AS cc ON cxref.catid = cc.id' .
					' LEFT JOIN #__sections AS s ON s.id = cc.section AND s.scope = "com_mymuse"' .
					' LEFT JOIN #__users AS u ON u.id = a.created_by' .
					' LEFT JOIN #__groups AS g ON a.access = g.id'.

					$where;
			$this->_db->setQuery($query);
			$this->_product = $this->_db->loadObject();

			if ( ! $this->_product ) {
				return false;
			}

			if($this->_product->publish_down == $this->_db->getNullDate()) {
				$this->_product->publish_down = JText::_('MYMUSE NEVER');
			}
			
			// add any artist(s)
			//$store->my_section_artists  $catid
			//$store->my_section_catalog
			
			//main artist
			$query = "SELECT * from #__mymuse_categories WHERE id='".$this->_product->artistid."'";
			$this->_db->setQuery($query);
			$this->_product->artist = $this->_db->loadObject();
			
			$catid 		= $this->_product->artistid;
			$sectionid 	= $this->_product->artist->section;
			$this->_product->artist->link = myMuseHelperRoute::getCategoryRoute($catid, $sectionid, $params);
			
			
			global $store;

			//catalog categories
			$query = "SELECT xref.catid, c.title, c.section
			FROM #__mymuse_product_category_xref as xref, #__mymuse_categories as c 
			WHERE c.id=xref.catid
			AND xref.product_id=".$this->_id."
			AND c.section='".$store->my_section_catalog."'";
			$this->_db->setQuery($query);
			$this->_product->categorynames = $this->_db->loadObjectList();
			for($i=0;$i<count($this->_product->categorynames);$i++){
				$catid 		= $this->_product->categorynames[$i]->catid;
				$sectionid 	= $this->_product->categorynames[$i]->section;
				$this->_product->categorynames[$i]->link = myMuseHelperRoute::getCategoryRoute($catid, $sectionid, $params);
			}

			//other artists
			$query = "SELECT xref.catid, c.title, c.section
			FROM #__mymuse_product_category_xref as xref, #__mymuse_categories as c 
			WHERE c.id=xref.catid
			AND xref.product_id=".$this->_id."
			AND c.section='".$store->my_section_artists."'";
			$this->_db->setQuery($query);
			$this->_product->artistnames = $this->_db->loadObjectList();
			for($i=0;$i<count($this->_product->categorynames);$i++){
				$catid 		= $this->_product->categorynames[$i]->catid;
				$sectionid 	= $this->_product->categorynames[$i]->section;
				$this->_product->categorynames[$i]->link = myMuseHelperRoute::getCategoryRoute($catid, $sectionid, $params);
			}
			
			//put the text together?
			if(!$params->get('product_show_intro_text')){
				$this->_product->introtext = $this->_product->introtext.$this->_product->fulltext;
			}else{
				$this->_product->readmorelink = myMuseHelperRoute::getProductRoute($this->_product->id, $this->_product->artistid,$params->get('my_section_artists'));
				$this->_product->readmorelink .= "#readmore";
			}
			
			
			// get price
			$this->_product->price = $this->getPrice($this->_product);
			
			// get child tracks with prices
			$query = "SELECT id,title, parentid, product_physical, product_downloadable, product_allfiles, product_sku,
			product_made_date, product_special, product_discount, product_package_ordering, product_package,file_length,
			file_name,file_preview,file_type 
			FROM #__mymuse_product 
			WHERE parentid='".$this->_id."'
			AND product_downloadable = 1
			ORDER BY ordering
			";
			$this->_db->setQuery($query);
			$tracks = $this->_db->loadObjectList();
			
			$MyMuseStore	=& MyMuse::getObject('store','models');
			$player 		=  $MyMuseStore->_store->player;
			
			// set up flash previews
			if(count($tracks)){
				while (list($i,$track)= each( $tracks )){
					$tracks[$i]->price = $this->getPrice($track);
				}
				require_once(JPATH_COMPONENT.DS.'assets'.DS.'players'.DS.'mp3players.php');
				foreach($tracks as $track){
					if($track->file_preview){
						$flash = '<!-- Begin Flash Player -->';
						$flash .= mp3players::getplayer($player, $params->get('my_preview_dir').DS.$track->file_preview);
						$flash .= '<!-- End Flash Player -->';
					}else{
						$flash = '';
					}
					$track->flash = $flash;
				}
				 
			}

			$this->_product->tracks = $tracks;

			// get child items with prices
			$query = "SELECT * FROM #__mymuse_product as p
			WHERE p.parentid='".$this->_id."'
			AND product_downloadable = 0
			ORDER BY ordering
			";
			$this->_db->setQuery($query);
			$items = $this->_db->loadObjectList();

			
			//attributes
			$query = 'SELECT * from #__mymuse_product_attribute_sku WHERE 
			product_parent_id='.$this->_product->id.'
			ORDER BY ordering';
			$this->_db->setQuery($query);
			$this->_product->attribute_sku = $this->_db->loadObjectList();
			
			while (list($i,$item)= each( $items )){
				foreach($this->_product->attribute_sku as $a_sku){
					$query = 'SELECT attribute_value from #__mymuse_product_attribute WHERE product_id='.$item->id.'
					AND product_attribute_sku_id='.$a_sku->id;
					$this->_db->setQuery($query);
					$items[$i]->attributes[$a_sku->name] = $this->_db->loadResult();
				}
	
					
				$query = 'SELECT a.*,b.name from #__mymuse_product_attribute as a 
				LEFT JOIN #__mymuse_product_attribute_sku as b on b.id=a.product_attribute_sku_id
				WHERE a.product_id='.$item->id;
				
				$this->_db->setQuery($query);
				$tmp[$item->id] = $this->_db->loadObjectList();
				foreach($tmp[$item->id] as $att){
					$attributes[$item->id][$att->product_attribute_sku_id] = $att;
				}

				$items[$i]->price = $this->getPrice($item);

			}
			//$this->_product->attributes = $attributes;
			$this->_product->items = $items;
			
			// get list of review articles
			$contents = array();
			$this->_product->reviews = array();
			$query = 'SELECT x.content_id,c.title' .
			' FROM #__mymuse_product_content_xref as x' .
			' LEFT JOIN #__content as c on c.id=x.content_id ' .
			' WHERE x.product_id='. $this->_product->id 
			;
			//echo  $query;
			$this->_db->setQuery($query);
			if($revs = $this->_db->loadObjectList()){
				$this->_product->reviews = $revs;
			}
			
			
			
			
			
			// update hits
			$this->hit($this->_id);
			return true;
		}
		return true;
	}

	/**
	 * Method to load content product parameters
	 *
	 * @access	private
	 * @return	void

	 */
	function _loadProductParams()
	{
		global $mainframe;

		// Get the page/component configuration
		$params = clone($mainframe->getParams('com_mymuse'));

		// Merge product parameters into the page configuration
		$aparams = new JParameter($this->_product->attribs);
		$params->merge($aparams);

		// Set the popup configuration option based on the request
		$pop = JRequest::getVar('pop', 0, '', 'int');
		$params->set('popup', $pop);

		// Are we showing introtext with the product
		if (!$params->get('show_intro') && !empty($this->_product->fulltext)) {
			$this->_product->text = $this->_product->fulltext;
		} else {
			$this->_product->text = $this->_product->introtext . chr(13).chr(13) . $this->_product->fulltext;
		}

		// Set the product object's parameters
		$this->_product->parameters = & $params;
	}

	/**
	 * Method to build the WHERE clause of the query to select a content product
	 *
	 * @access	private
	 * @return	string	WHERE clause
	 * @since	1.5
	 */
	function _buildContentWhere()
	{
		global $mainframe;

		$user		=& JFactory::getUser();
		$aid		= (int) $user->get('aid', 0);

		$jnow		=& JFactory::getDate();
		$now		= $jnow->toMySQL();
		$nullDate	= $this->_db->getNullDate();

		/*
		 * First thing we need to do is assert that the product is the one
		 * we are looking for and we have access to it.
		 */
		$where = ' WHERE a.id = '. (int) $this->_id;
//		$where .= ' AND a.access <= '. (int) $aid;

		if (!$user->authorize('com_mymuse', 'edit', 'content', 'all'))
		{
			$where .= ' AND ( ';
			$where .= ' ( a.created_by = ' . (int) $user->id . ' ) ';
			$where .= '   OR ';
			$where .= ' ( a.state = 1' .
					' AND ( a.publish_up = '.$this->_db->Quote($nullDate).' OR a.publish_up <= '.$this->_db->Quote($now).' )' .
					' AND ( a.publish_down = '.$this->_db->Quote($nullDate).' OR a.publish_down >= '.$this->_db->Quote($now).' )';
			$where .= '   ) ';
			$where .= '   OR ';
			$where .= ' ( a.state = -1 ) ';
			$where .= ' ) ';
		}
		$catid_str = JRequest::getVar('catid', null, 'default', 'string');
		if(preg_match("/:/",$catid_str)){
			list($catid,$string) = explode(":",$catid_str);
			if($catid){
				$catid= (int)$catid;
				$where .= " AND cc.id=$catid ";
			}
		}
		return $where;
	}
	
		/**
     * getAttributes
     * 
     * @param int $item_id
     * @param int $product_id
     * @param string $attribute_name
     * @return object
     */
  function getAttributes($item_id="",$product_id="",$attribute_name="") {

  	$db	= & $this->_db;
  	
    if ($item_id and $product_id) {
      $q  = "SELECT * FROM #__mymuse_product_attribute as pa, #__mymuse_product_attribute_sku as pas  \n";
      $q .= "WHERE pa.product_id = '$item_id'  \n";
      $q .= "AND pas.product_parent_id =$product_id  \n";
      if ($attribute_name) {
        $q .= "AND pa.attribute_name"." = '$attribute_name'  \n";
      }
      $q .= "AND pa.product_attribute_sku_id=pas.id  \n";
      $q .= "ORDER BY ordering, pa.attribute_name  \n";
    } elseif ($item_id) {
      $q  = "SELECT * FROM #__mymuse_product_attribute  \n";
      $q .= "WHERE product_id = $item_id ";
      if ($attribute_name) {
        $q .= "AND attribute_name = '$attribute_name'  \n";
      }
    } elseif ($product_id) {
      $q  = "SELECT * FROM #__mymuse_product_attribute_sku  \n";
      $q .= "WHERE product_parent_id =$product_id  \n";
      if ($attribute_name) {
        $q .= "AND #__mymuse_product_attribute.attribute_name = '$attribute_name'  \n";
      }
      $q .= " ORDER BY ordering,attribute_name \n";
      //$q .= " ORDER BY attribute_list ";
    } else {
      $this->error = JText::_("MYMUSE ERROR GET ATTRIBUTE");
      return false;
    }

	$db->setQuery($q);
	$res = $db->loadObjectList();
    return $res;
  }
	
   /**
     * getPrice
     * 
     * @param object $product
     * @return mixed Array or false: array [product_price] [special_shopper_group] [product_discount] [product_shopper_group_discount]
     */
	function getPrice(&$product) {

		global $mainframe,$params,$shopper;

		$db	= & JFactory::getDBO();
		
		$shopper_group_discount = 0;
		$discount = 0;
		$price_info = array();
		$price_info["item"]=false;

		$default_shopper_group_id = $params->get("my_default_shopper_group_id");
		$product_id = $product->id;

		// see if this product has a discount
		$discount = $product->product_discount;

		// Get the shopper group id for this shopper
		$shopper_group_id = @$shopper->shopper_group_id;
		if($shopper_group_id == ""){
			$shopper_group_id = $default_shopper_group_id;
		}

		// Get the product_parent_id for this product/item
		$product_parent_id = 0;
		if(isset($product->parentid)){
			$product_parent_id = $product->parentid;
			if($product_parent_id > 0){
				$price_info["item"]=True;
			}
		}
		// Get the shopper group discount
		if(@$shopper_group_id){
			$q = "SELECT discount FROM #__mymuse_shopper_group WHERE  \n";
			$q .= "shopper_group_id='";
			$q .= $shopper_group_id . "'";
			$db->setQuery($q);
			$shopper_group_discount  = $db->loadResult();
		}

		// DEBUG
		//echo "product:$product_id disount:$discount, shopper group id = $shopper_group_id, shopper group disount = $shopper_group_discount<BR>";

		// Getting prices
		//
		// If the shopper group has a price then show it, otherwise
		// show the default price.


		// IT'S FOR A SPECIAL SHOPPER GROUP?
		if($shopper_group_id != $default_shopper_group_id){
			$q = "SELECT * from #__mymuse_product_price WHERE product_id='$product_id' AND  \n";
			$q .= "shopper_group_id='$shopper_group_id'";
			$db->setQuery($q);
			$res = $db->loadObject();
			if ($res->product_price && $res->product_price > 0) {
				$price_info["special_shopper_group"] = True;
				$price_info["product_price"]=$res->product_price - ($res->product_price*$shopper_group_discount/100)-$discount;
				$price_info["product_discount"] = $discount;
				$price_info["product_shopper_group_discount"] = $shopper_group_discount;
				$price_info["product_price"] = sprintf("%.2f",$price_info["product_price"]);
				return $price_info;
			}
		}


		// MAYBE IT'S AN ITEM, FIRST SEE IF IT HAS ITS OWN PRICE
		if(isset($product_parent_id) && $product_parent_id > 0){
			$q = "SELECT * from #__mymuse_product_price WHERE product_id='$product_id' AND  \n";
			$q .= "shopper_group_id='$shopper_group_id'";
			$db->setQuery($q);
			$res = $db->loadObject();
			//if (isset($res->product_price) && $res->product_price > 0) {
			if (isset($res->product_price)) {
				$price_info["product_price"]=$res->product_price - ($res->product_price*$shopper_group_discount/100)-$discount;
				$price_info["product_discount"] = $discount;
				$price_info["product_shopper_group_discount"] = $shopper_group_discount;
				$price_info["product_price"] = sprintf("%.2f",$price_info["product_price"]);
				if($price_info["product_price"] == 0.00){
					$price_info["product_price"] = "FREE";
				}
				return $price_info;
			}
		}

		 
		// Get default price
		$q = "SELECT * from #__mymuse_product_price WHERE product_id='$product_id' AND  \n";
		$q .= "shopper_group_id='$default_shopper_group_id'";
		$db->setQuery($q);
		$res = $db->loadObject();
		if ($res && $res->product_price > 0) {
			$price_info["default"] = True;
			$price_info["product_price"]=$res->product_price - ($res->product_price*$shopper_group_discount/100)-$discount;
			$price_info["product_discount"] = $discount;
			$price_info["product_shopper_group_discount"] = $shopper_group_discount;
			$price_info["product_price"] = sprintf("%.2f",$price_info["product_price"]);
			return $price_info;
		}

		// Maybe its an item with no price, check again with product_parent_id
		// first with shopper group ID
		$q = "SELECT * from #__mymuse_product_price WHERE product_id='$product_parent_id' AND  \n";
		$q .= "shopper_group_id='$shopper_group_id'";
		$db->setQuery($q);
		$res = $db->loadObject();
		if (isset($res->product_price) && $res->product_price > 0) {
			$price_info["item_uses_parent"] = True;
			$price_info["product_price"]=$res->product_price - ($res->product_price*$shopper_group_discount/100)-$discount;
			$price_info["product_discount"] = $discount;
			$price_info["product_shopper_group_discount"] = $shopper_group_discount;
			$price_info["product_price"] = sprintf("%.2f",$price_info["product_price"]);
			return $price_info;
		}
		// now parent with default shopper group
		$q = "SELECT * from #__mymuse_product_price WHERE product_id='$product_parent_id' AND  \n";
		$q .= "shopper_group_id='$default_shopper_group_id'";
		$db->setQuery($q);
		$res = $db->loadObject();
		if (isset($res->product_price) && $res->product_price > 0) {
			$price_info["parent_default"] = True;
			$price_info["product_price"]=$res->product_price - ($res->product_price*$shopper_group_discount/100)-$discount;
			$price_info["product_discount"] = $discount;
			$price_info["product_shopper_group_discount"] = $shopper_group_discount;
			$price_info["product_price"] = sprintf("%.2f",$price_info["product_price"]);
			return $price_info;
		}
		
		// No price found, must be FREE

		
		return False;
  	}
  	
}
	